#include "sq.h"

#include <dirent.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <io.h>
#include <dos.h>
#include <mntent.h>
#include <dir.h>

/***
	The interface to the directory primitive is path based.
	That is, the client supplies a Squeak string describing
	the path to the directory on every call. To avoid traversing
	this path on every call, a cache is maintained of the last
	path seen, along with the Mac volume and folder reference
	numbers corresponding to that path.

NOTES:
*    1) Caching is not supported because Andreas didn't support it. ;)
	Because there was no noticeable delay."

***/

/*** Constants ***/
#define ENTRY_FOUND     0
#define NO_MORE_ENTRIES 1
#define BAD_PATH        2
#define SEARCHATTRIB	FA_HIDDEN | FA_SYSTEM | FA_DIREC | FA_ARCH | FA_RDONLY
#define DELIMITER '\\'

/*** Variables ***/
char lastPath[MAXPATHLEN+1];
int  lastPathValid = false;
int  lastIndex;
DIR *openDir;
static int areDrivesSet = 0;
static char allDrives[26] =
{ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 
};


/*** Functions ***/
int convertToSqueakTime(int unixTime);
int equalsLastPath(char *pathString, int pathStringLength);
int recordPath(char *pathString, int pathStringLength, int refNum, int volNum);
void setAllDrives();

int convertToSqueakTime(int unixTime)
{
  /* Squeak epoch is Jan 1, 1901.  Unix epoch is Jan 1, 1970: 17 leap years
     and 52 non-leap years later than Squeak. */
  return (unsigned long)unixTime + ((52*365UL + 17*366UL) * 24*60*60UL);
}

int dir_Create(char *pathString, int pathStringLength)
{
  /* Create a new directory with the given path. By default, this
     directory is created relative to the cwd. */
  char name[1024];
  int i;
  for (i = 0; i < pathStringLength; i++)
    name[i] = pathString[i];
  name[i] = 0; /* string terminator */
  return mkdir(name, 022) == 0;	/* rwxr-xr-x */
}

int dir_Delimitor(void)
{
  return DELIMITER;
}

int dir_Lookup(char *pathString, int pathStringLength, int index,
/* outputs: */ char *name, int *nameLength, int *creationDate, int *modificationDate,
	       int *isDirectory, int *sizeIfFile)
{
  /* Lookup the index-th entry of the directory with the given path, starting
     at the root of the file system. Set the name, name length, creation date,
     creation time, directory flag, and file size (if the entry is a file).
     Return:	0 	if a entry is found at the given index
     			1	if the directory has fewer than index entries
			2	if the given path has bad syntax or 
				does not reach a directory
  */

  char dosPath[MAXPATHLEN+1];
  struct ffblk findData;
  long findHandle;
  int i,nameLen,mask;
  char drive;
  struct mntent *m;
  FILE *f;


  /* default return values */
  *name             = 0;
  *nameLength       = 0;
  *creationDate     = 0;
  *modificationDate = 0;
  *isDirectory      = false;
  *sizeIfFile       = 0;

if (!areDrivesSet)
	setAllDrives();
  if(pathStringLength == 0)
    { /* we're at the top of the file system --- return possible drives */
//	if (!areDrivesSet) 
//		setAllDrives();
//	else
		{
		if ((index<0) | (index > 26))
			return NO_MORE_ENTRIES;
	 	for(i=0;i<26; i++)
			{
        		if(allDrives[i])
				{
				//printf ("Drive[%d] is 1 and index is %d\n", i, index);
	  			if(--index == 0)
					{
					name[0]           = 'A'+i;
					name[1]	      = ':';
				  	*nameLength       = 2;
				  	*creationDate     = 0;
				  	*modificationDate = 0;
				  	*isDirectory      = true;
				  	*sizeIfFile       = 0;
					return ENTRY_FOUND;
					}
				}
			}
		return NO_MORE_ENTRIES;
		}
	}
	
  for (i = 0; i < pathStringLength; i++)
    dosPath[i] = pathString[i];
  dosPath[pathStringLength] = 0;
  if(dosPath[pathStringLength-1] != DELIMITER)
    strcat(dosPath,"\\");
  strcat(dosPath,"*.*");
  findHandle = -1;
  for(i=0;i<index; i++)
    {
      if(findHandle == -1)
        findHandle = findfirst(dosPath,&findData, SEARCHATTRIB);
      if(findHandle == -1)
        return BAD_PATH;
      else if(findnext(&findData) == -1)
        {
	  _dos_close(findHandle);
	  return NO_MORE_ENTRIES;
	}
      nameLen = strlen(findData.ff_name);
      if (nameLen < 3 && findData.ff_name[0] == '.')
	if (nameLen == 1 || findData.ff_name[1] == '.')
	  i = i - 1; /* hack us back to the last index */
    }

  strcpy(name, findData.ff_name);
  *nameLength= strlen(name);

  /* use modification time instead (just like ls) */
  *creationDate= convertToSqueakTime(findData.ff_ftime & 0x1f) * 2;
  /* use status change time instead */
  *modificationDate= convertToSqueakTime(findData.ff_ftime & 0x1f) * 2;

  if (findData.ff_attrib & FA_DIREC)
    *isDirectory= true;
  else
    *sizeIfFile= findData.ff_fsize;

  return ENTRY_FOUND;
}



dir_SetMacFileTypeAndCreator(char *filename, int filenameSize,
			     char *fType, char *fCreator)
{
  /* dos files are untyped, and the creator is correct by default */
  return true;
}


/* Set the static variable allDrives to a list of the current
drives.  Although this doesn't work for every case it is much
quicker than calling getmntent() everytime it is needed.  getmntent()
takes around 1 to 2 seconds because of floppy A:. */
void setAllDrives()
	{
  struct mntent *m;
  FILE *f;
	char aChar;
	char ns[] = "";
	int index=-1;

	areDrivesSet = 1;
	f = (FILE *) setmntent(ns, ns);
	while ((m = getmntent(f)))
		{
//		if (!(strcmp(m->mnt_type,"fd")) | !(strcmp(m->mnt_type,"hd")))
			{
//			printf ("Type is %s\n", m->mnt_type);
			aChar= m->mnt_dir[0];
			index = aChar - 'a';
//			printf ("Drive %c index %d\n", aChar, index);
			if (index >= 0 & (index < 27))
				allDrives[index] = 1;
			}
		}
	endmntent(f);
	}